@@ -7,8 +7,7 @@ from typing import List, Optional, Dict, Any
 from datetime import datetime, timedelta
 from homeconnect_webthing.auth import Auth
 from homeconnect_webthing.eventstream import EventListener, ReconnectingEventStream
-from homeconnect_webthing.utils import print_duration
-
+from homeconnect_webthing.utils import print_duration, DailyRequestCounter
 
 
 DRYER = 'dryer'
@@ -46,7 +45,7 @@ class Targetdate:
 
 class Appliance(EventListener):
 
-    def __init__(self, device_uri: str, auth: Auth, name: str, device_type: str, haid: str, brand: str, vib: str, enumber: str):
+    def __init__(self, device_uri: str, auth: Auth, name: str, device_type: str, haid: str, brand: str, vib: str, enumber: str, request_counter: DailyRequestCounter):
         self._device_uri = device_uri
         self._auth = auth
         self.name = name
@@ -55,6 +54,7 @@ class Appliance(EventListener):
         self.brand = brand
         self.vib = vib
         self.enumber = enumber
+        self.request_counter = request_counter
         self.__value_changed_listeners = set()
         self.last_refresh = datetime.now() - timedelta(hours=9)
         self.remote_start_allowed = False
@@ -67,6 +67,9 @@ class Appliance(EventListener):
         self._operation = ""
         self._refresh(reason=self.name + " appliance initialized")
 
+    def id(self) -> str:
+        return self.haid
+
     @property
     def power(self):
         if len(self._power) > 0:
@@ -110,33 +113,34 @@ class Appliance(EventListener):
         for value_changed_listener in self.__value_changed_listeners:
             value_changed_listener()
 
-    def on_connected(self):
-        logging.info("refresh " + self.name + " state (new event stream has been established)")
+    def on_connected(self, event):
+        logging.info(self.name + " device has been connected")
         self._refresh(reason="on connected")
 
+    def on_disconnected(self, event):
+        logging.info(self.name + " device has been disconnected")
+
     def on_keep_alive_event(self, event):
-        logging.debug("keep alive event")
+        self.__notify_listeners()
 
     def on_notify_event(self, event):
-        logging.debug("notify event: " + str(event.data))
+        logging.debug(self.name + " notify event: " + str(event.data))
         self._on_value_changed_event(event)
 
     def on_status_event(self, event):
-        logging.debug("status event: " + str(event.data))
+        logging.debug(self.name + " status event: " + str(event.data))
         self._on_value_changed_event(event)
 
     def _on_event_event(self, event):
-        logging.debug("event event: " + str(event.data))
-        pass
+        logging.debug(self.name + " event event: " + str(event.data))
 
     def _on_value_changed_event(self, event):
-        if event.id == self.haid:
-            try:
-                data = json.loads(event.data)
-                self._on_values_changed(data.get('items', []), "updated")
-                self.__notify_listeners()
-            except Exception as e:
-                logging.warning("error occurred by handling event " + str(event), e)
+        try:
+            data = json.loads(event.data)
+            self._on_values_changed(data.get('items', []), "updated")
+            self.__notify_listeners()
+        except Exception as e:
+            logging.warning("error occurred by handling event " + str(event), e)
 
     def _on_values_changed(self, changes: List[Any], ops: str = "updated"):
         for record in changes:
@@ -206,6 +210,7 @@ class Appliance(EventListener):
 
     def _perform_get(self, path:str, ignore_error_codes: List[int] = None) -> Dict[str, Any]:
         uri = self._device_uri + path
+        self.request_counter.inc()
         response = requests.get(uri, headers={"Authorization": "Bearer " + self._auth.access_token}, timeout=5000)
         if is_success(response.status_code):
             return response.json()
@@ -218,6 +223,7 @@ class Appliance(EventListener):
 
     def _perform_put(self, path:str, data: str, max_trials: int = 3, current_trial: int = 1):
         uri = self._device_uri + path
+        self.request_counter.inc()
         response = requests.put(uri, data=data, headers={"Content-Type": "application/json", "Authorization": "Bearer " + self._auth.access_token}, timeout=5000)
         if not is_success(response.status_code):
             logging.warning("error occurred by calling PUT " + uri + " " + data)
@@ -227,6 +233,7 @@ class Appliance(EventListener):
                 logging.warning("waiting " + str(delay) + " sec for retry")
                 sleep(delay)
                 self._perform_put(path, data, max_trials, current_trial+1)
+            response.raise_for_status()
 
     @property
     def __fingerprint(self) -> str:
@@ -245,7 +252,7 @@ class Appliance(EventListener):
 
 class Dishwasher(Appliance):
 
-    def __init__(self, device_uri: str, auth: Auth, name: str, device_type: str, haid: str, brand: str, vib: str, enumber: str):
+    def __init__(self, device_uri: str, auth: Auth, name: str, device_type: str, haid: str, brand: str, vib: str, enumber: str, request_counter: DailyRequestCounter):
         self.__program_start_in_relative_sec = 0
         self.program_remaining_time_sec = 0
         self.__program_active = ""
@@ -254,7 +261,7 @@ class Dishwasher(Appliance):
         self.program_vario_speed_plus = ""
         self.program_energy_forecast_percent = 0
         self.program_water_forecast_percent = 0
-        super().__init__(device_uri, auth, name, device_type, haid, brand, vib, enumber)
+        super().__init__(device_uri, auth, name, device_type, haid, brand, vib, enumber, request_counter)
 
     def _on_value_changed(self, key: str, value: str, ops: str) -> bool:
         if key == 'BSH.Common.Root.ActiveProgram':
@@ -314,7 +321,7 @@ class Dishwasher(Appliance):
                 self._perform_put("/programs/active", json.dumps(data, indent=2), max_trials=3)
                 logging.info(self.name + " program " + self.program_selected + " starts in " + print_duration(remaining_secs_to_wait) + " (program duration " + print_duration(self.program_remaining_time_sec) + ")")
             except Exception as e:
-                logging.warning("error occurred by starting " + self.name, e)
+                logging.warning("error occurred by starting " + self.name + " " + str(e))
         else:
             logging.warning("ignoring start command. " + self.name + " is in state " + self._operation)
         self._refresh(reason=self.name + " write start date post-refresh")
@@ -323,15 +330,22 @@ class Dishwasher(Appliance):
 
 class Dryer(Appliance):
 
-    def __init__(self, device_uri: str, auth: Auth, name: str, device_type: str, haid: str, brand: str, vib: str, enumber: str):
+    def __init__(self, device_uri: str, auth: Auth, name: str, device_type: str, haid: str, brand: str, vib: str, enumber: str, request_counter: DailyRequestCounter):
         self.__program_finish_in_relative_sec = 0
-        super().__init__(device_uri, auth, name, device_type, haid, brand, vib, enumber)
         self.child_lock = False
         self.program_gentle = False
         self.__program_drying_target = ""
         self.__program_drying_target_adjustment = ""
         self.__program_wrinkle_guard = ""
         self.__program_duration_sec = 0
+        super().__init__(device_uri, auth, name, device_type, haid, brand, vib, enumber, request_counter)
+
+    @property
+    def program_duration_sec(self) -> int:
+        if self.__program_finish_in_relative_sec > 0:
+            return self.__program_finish_in_relative_sec
+        else:
+            return 3 * 60 * 60
 
     @property
     def program_wrinkle_guard(self) -> str:
@@ -401,26 +415,24 @@ class Dryer(Appliance):
                     targetdate.stepsize = constrains.get("stepsize", targetdate.stepsize)
 
     def write_end_date(self, end_date: str):
+        self._refresh(notify=False, reason=self.name + " write end date pre-refresh")
         if self._operation in ["BSH.Common.EnumType.OperationState.Ready", '']:
-            program_duration_sec = self.__program_finish_in_relative_sec
-            start_date = datetime.fromisoformat(end_date) - timedelta(seconds=program_duration_sec)
-            logging.info("end_date " + str(end_date))
-            logging.info("program_duration_sec " + str(program_duration_sec))
-            logging.info("computed start_date " + str(start_date.isoformat()))
+            start_date = datetime.fromisoformat(end_date) - timedelta(seconds=self.program_duration_sec)
+            logging.info("WRITE_END_DATE end_date " + str(end_date))
+            logging.info("WRITE_END_DATE program_duration_sec " + str(self.program_duration_sec))
+            logging.info("WRITE_END_DATE computed start_date " + str(start_date.isoformat()))
             self.write_start_date(start_date.isoformat())
 
     def write_start_date(self, start_date: str):
         self._refresh(notify=False, reason=self.name + " write start date pre-refresh")
         if self._operation in ["BSH.Common.EnumType.OperationState.Ready", '']:
-            self.__program_duration_sec = self.__program_finish_in_relative_sec
             targetdate = Targetdate(start_date)
             self.__update_target_date_options(targetdate)
-            program_duration_sec = self.__program_finish_in_relative_sec
-            remaining_secs_to_finish = targetdate.remaining_secs_to_finish(program_duration_sec)
-            logging.info("start_date " + str(start_date))
-            logging.info("remaining secs to start " + str(targetdate.remaining_secs_to_start()))
-            logging.info("program_finish_in_relative_sec " + str(self.__program_finish_in_relative_sec))
-            logging.info("remaining_secs_to_finish " + str(remaining_secs_to_finish))
+            remaining_secs_to_finish = targetdate.remaining_secs_to_finish(self.program_duration_sec)
+            logging.info("WRITE_START_DATE start_date " + str(start_date))
+            logging.info("WRITE_START_DATE remaining secs to start " + str(targetdate.remaining_secs_to_start()))
+            logging.info("WRITE_START_DATE program_duration_sec " + str(self.program_duration_sec))
+            logging.info("WRITE_START_DATE remaining_secs_to_finish " + str(remaining_secs_to_finish))
 
             data = {
                 "data": {
@@ -434,20 +446,20 @@ class Dryer(Appliance):
             }
             try:
                 self._perform_put("/programs/active", json.dumps(data, indent=2), max_trials=3)
-                logging.info(self.name + " program " + "self.program_selected" + " starts in " + print_duration(targetdate.remaining_secs_to_start()) + " (duration: " + print_duration(program_duration_sec) + ")")
+                logging.info(self.name + " program " + "self.program_selected" + " starts in " + print_duration(targetdate.remaining_secs_to_start()) + " (duration: " + print_duration(self.program_duration_sec) + ")")
             except Exception as e:
-                logging.warning("error occurred by starting " + self.name, e)
+                logging.warning("error occurred by starting " + self.name + " " + str(e))
         else:
             logging.warning("ignoring start command. " + self.name + " is in state " + self._operation)
         self._refresh(reason=self.name + " start date updated post-refresh")
 
 
 
-def create_appliance(uri: str, auth: Auth, name: str, device_type: str, haid: str, brand: str, vib: str, enumber: str) -> Optional[Appliance]:
+def create_appliance(uri: str, auth: Auth, name: str, device_type: str, haid: str, brand: str, vib: str, enumber: str, request_counter: DailyRequestCounter) -> Optional[Appliance]:
     if device_type.lower() == DISHWASHER:
-        return Dishwasher(uri, auth, name, device_type, haid, brand, vib, enumber)
+        return Dishwasher(uri, auth, name, device_type, haid, brand, vib, enumber, request_counter)
     elif device_type.lower() == DRYER:
-        return Dryer(uri, auth, name, device_type, haid, brand, vib, enumber)
+        return Dryer(uri, auth, name, device_type, haid, brand, vib, enumber, request_counter)
     else:
         return None
 
@@ -460,6 +472,7 @@ class HomeConnect:
     def __init__(self, refresh_token: str, client_secret: str):
         self.notify_listeners: List[EventListener] = list()
         self.auth = Auth(refresh_token, client_secret)
+        self.request_counter = DailyRequestCounter()
         Thread(target=self.__start_consuming_events, daemon=True).start()
 
     # will be called by a background thread
@@ -468,32 +481,42 @@ class HomeConnect:
         ReconnectingEventStream(HomeConnect.API_URI + "/homeappliances/events",
                                 self.auth,
                                 self,
+                                self.request_counter,
                                 read_timeout_sec=3*60,
-                                max_lifetime_sec=170*60).consume()
+                                max_lifetime_sec=7*60*60).consume()
 
-    def on_connected(self):
-        for notify_listener in self.notify_listeners:
-            notify_listener.on_connected()
+    def __is_assigned(self, notify_listener: EventListener, event):
+        return event is None or event.id is None or event.id == notify_listener.id()
 
-    def on_disconnected(self):
+    def on_connected(self, event):
         for notify_listener in self.notify_listeners:
-            notify_listener.on_disconnected()
+            if self.__is_assigned(notify_listener, event):
+                notify_listener.on_connected(event)
+
+    def on_disconnected(self, event):
+        for notify_listener in self.notify_listeners:
+            if self.__is_assigned(notify_listener, event):
+                notify_listener.on_disconnected(event)
 
     def on_keep_alive_event(self, event):
         for notify_listener in self.notify_listeners:
-            notify_listener.on_keep_alive_event(event)
+            if self.__is_assigned(notify_listener, event):
+                notify_listener.on_keep_alive_event(event)
 
     def on_notify_event(self, event):
         for notify_listener in self.notify_listeners:
-            notify_listener.on_notify_event(event)
+            if self.__is_assigned(notify_listener, event):
+                notify_listener.on_notify_event(event)
 
     def on_status_event(self, event):
         for notify_listener in self.notify_listeners:
-            notify_listener.on_status_event(event)
+            if self.__is_assigned(notify_listener, event):
+                notify_listener.on_status_event(event)
 
     def on_event_event(self, event):
         for notify_listener in self.notify_listeners:
-            notify_listener.on_event_event(event)
+            if self.__is_assigned(notify_listener, event):
+                notify_listener.on_event_event(event)
 
     def appliances(self) -> List[Appliance]:
         uri = HomeConnect.API_URI + "/homeappliances"
@@ -510,7 +533,8 @@ class HomeConnect:
                                           homeappliances['haId'],
                                           homeappliances['brand'],
                                           homeappliances['vib'],
-                                          homeappliances['enumber'])
+                                          homeappliances['enumber'],
+                                          self.request_counter)
                 if device is not None:
                     self.notify_listeners.append(device)
                     devices.append(device)
